@@ -49,21 +49,23 @@
 #include <util/fileops.h>
 #include <util/functions.h>
 #include <util/waitjob.h>
-
+#include <plugin/pluginmanager.h>
 #include <groups/groupmanager.h>
 #include <groups/group.h>
+#include <torrent/jobqueue.h>
 
 #ifdef ENABLE_DHT_SUPPORT
 #include <dht/dht.h>
 #endif
 
 #include "settings.h"
-#include "pluginmanager.h"
 #include "core.h"
 #include "fileselectdlg.h"
 #include "missingfilesdlg.h"
 #include "gui.h"
 #include "torrentmigratordlg.h"
+#include "scanlistener.h"
+
 
 
 using namespace bt;
@@ -83,7 +85,7 @@ namespace kt
 		connect(qman, SIGNAL(lowDiskSpace(bt::TorrentInterface*, bool)),
 				this, SLOT(onLowDiskSpace(bt::TorrentInterface*, bool)));
 		
-		data_dir = Settings::tempDir().path();
+		data_dir = Settings::tempDir().toLocalFile();
 		bool dd_not_exist = !bt::Exists(data_dir);
 		if (data_dir == QString::null || dd_not_exist)
 		{
@@ -101,7 +103,6 @@ namespace kt
 			data_dir += bt::DirSeparator();
 
 		connect(&update_timer,SIGNAL(timeout()),this,SLOT(update()));
-		update_timer.start(CORE_UPDATE_INTERVAL);
 		
 		Uint16 port = Settings::port();
 		if (port == 0)
@@ -172,7 +173,7 @@ namespace kt
 		setMaxSeeds(Settings::maxSeeds());
 		setKeepSeeding(Settings::keepSeeding());
 		
-		QString tmp = Settings::tempDir().path();
+		QString tmp = Settings::tempDir().toLocalFile();
 		if (tmp.isEmpty())
 			tmp = kt::DataDir();
 		
@@ -189,26 +190,31 @@ namespace kt
 		pman->loadPluginList();
 	}
 
-	bool Core::init(TorrentControl* tc,const QString & group,bool silently)
+	bool Core::init(TorrentControl* tc,const QString & group,const QString & location,bool silently)
 	{
-		bool user = false;
 		bool start_torrent = false;
 		bool skip_check = false;
+		
+		if (Settings::maxRatio() > 0)
+			tc->setMaxShareRatio(Settings::maxRatio());
+		if (Settings::maxSeedTime() > 0)
+			tc->setMaxSeedTime(Settings::maxSeedTime());
 
 		if (!silently)
 		{
-			if (!gui->selectFiles(tc,&user,&start_torrent,group,&skip_check))
+			if (!gui->selectFiles(tc,&start_torrent,group,location,&skip_check))
 			{
 				// Cleanup tor dir
 				QString dir = tc->getTorDir();
 				if (bt::Exists(dir))
 					bt::Delete(dir,true);
-				delete tc;
+				delete tc;	
 				return false;
 			}
 		}
 		else
 		{
+			start_torrent = true;
 			// add torrent to group if necessary
 			Group* g = gman->find(group);
 			if (g)
@@ -263,20 +269,15 @@ namespace kt
 		if (tc->hasExistingFiles())
 		{
 			if (!skip_check)
-				gui->dataScan(tc,false,true,QString::null);
+				doDataCheck(tc);
 			else
 				tc->markExistingFilesAsDownloaded();
 		}
 			
 		tc->setPreallocateDiskSpace(true);
-
-		if (Settings::maxRatio() > 0)
-			tc->setMaxShareRatio(Settings::maxRatio());
-		if (Settings::maxSeedTime() > 0)
-			tc->setMaxSeedTime(Settings::maxSeedTime());
 		
 		torrentAdded(tc);
-		qman->torrentAdded(tc,user,start_torrent);
+		qman->torrentAdded(tc,start_torrent);
 		//now copy torrent file to user specified dir if needed
 		if(Settings::useTorrentCopyDir())
 		{
@@ -285,7 +286,7 @@ namespace kt
 				torFile += bt::DirSeparator();
 			
 			torFile += "torrent";
-			QString destination = Settings::torrentCopyDir().path();
+			QString destination = Settings::torrentCopyDir().toLocalFile();
 			if(!destination.endsWith(bt::DirSeparator()))
 				destination += bt::DirSeparator();
 			
@@ -295,7 +296,7 @@ namespace kt
 		return true;
 	}
 	
-	bool Core::load(const QByteArray & data,const QString & dir,const QString & group,bool silently, const KUrl& url)
+	bool Core::loadFromData(const QByteArray & data,const QString & dir,const QString & group,bool silently, const KUrl& url)
 	{
 		QString tdir = findNewTorrentDir();
 		TorrentControl* tc = 0;
@@ -303,11 +304,10 @@ namespace kt
 		{
 			Out(SYS_GEN|LOG_NOTICE) << "Loading torrent from data " << endl;
 			tc = new TorrentControl();
-			tc->init(qman, data, tdir, dir, 
-					 Settings::useSaveDir() ? Settings::saveDir().path() : QString());
+			tc->init(qman, data, tdir, dir);
 			tc->setLoadUrl(url);
 			
-			if(!init(tc,group,silently))
+			if(!init(tc,group,dir,silently))
 				loadingFinished(url, false, true);
 			else
 				loadingFinished(url, true, false);
@@ -336,7 +336,7 @@ namespace kt
 		}
 	}
 
-	bool Core::load(const QString & target,const QString & dir,const QString & group,bool silently)
+	bool Core::loadFromFile(const QString & target,const QString & dir,const QString & group,bool silently)
 	{
 		QString tdir = findNewTorrentDir();
 		TorrentControl* tc = 0;
@@ -344,11 +344,10 @@ namespace kt
 		{
 			Out(SYS_GEN|LOG_NOTICE) << "Loading file " << target << endl;
 			tc = new TorrentControl();
-			tc->init(qman, target, tdir, dir, 
-				 Settings::useSaveDir() ? Settings::saveDir().path() : QString());
+			tc->init(qman, target, tdir, dir);
 			tc->setLoadUrl(KUrl(target));
 			
-			init(tc,group,silently);
+			init(tc,group,dir,silently);
 			startUpdateTimer();
 			return true;
 		}
@@ -388,10 +387,7 @@ namespace kt
 		else
 		{
 			// load in the file (target is always local)
-			QString dir = Settings::saveDir().path();
-			if (!Settings::useSaveDir() ||  dir.isNull())
-				dir = QDir::homePath();
-			
+			QString dir = locationHint();
 			QString group;
 			if (add_to_groups.contains(j))
 			{
@@ -399,7 +395,7 @@ namespace kt
 				add_to_groups.remove(j);
 			}
 		
-			if (dir != QString::null && load(j->data(),dir,group,false, j->url()))
+			if (dir != QString::null && loadFromData(j->data(),dir,group,false, j->url()))
 				loadingFinished(j->url(),true,false);
 			else
 				loadingFinished(j->url(),false,true);
@@ -410,12 +406,9 @@ namespace kt
 	{
 		if (url.isLocalFile())
 		{
-			QString path = url.path(); 
-			QString dir = Settings::saveDir().path();
-			if (!Settings::useSaveDir()  || dir.isNull())
-				dir =  QDir::homePath();
-		
-			if (dir != QString::null && load(path,dir,group,false))
+			QString path = url.toLocalFile();
+			QString dir = locationHint();
+			if (dir != QString::null && loadFromFile(path,dir,group,false))
 				loadingFinished(url,true,false);
 			else
 				loadingFinished(url,false,true);
@@ -447,7 +440,7 @@ namespace kt
 			if (custom_save_locations.contains(j))
 			{
 				// we have a custom save location so save to that
-				dir = custom_save_locations[j].path();
+				dir = custom_save_locations[j].toLocalFile();
 				custom_save_locations.remove(j);
 			}
 			else if (!Settings::useSaveDir())
@@ -459,7 +452,7 @@ namespace kt
 			}
 			else
 			{
-				dir = Settings::saveDir().path();
+				dir = Settings::saveDir().toLocalFile();
 			}
 			
 			QString group;
@@ -470,7 +463,7 @@ namespace kt
 			}
 				
 			
-			if (dir != QString::null && load(j->data(),dir,group,true,j->url()))
+			if (dir != QString::null && loadFromData(j->data(),dir,group,true,j->url()))
 				loadingFinished(j->url(),true,false);
 			else
 				loadingFinished(j->url(),false,false);
@@ -481,16 +474,10 @@ namespace kt
 	{
 		if (url.isLocalFile())
 		{
-			QString path = url.path(); 
-			QString dir = Settings::saveDir().path();
-			if (!Settings::useSaveDir())
-			{
-				Out(SYS_GEN|LOG_NOTICE) << "Cannot load " << path << " silently, default save location not set !" << endl;
-				Out(SYS_GEN|LOG_NOTICE) << "Using home directory instead !" << endl;
-				dir = QDir::homePath();
-			}
+			QString path = url.toLocalFile(); 
+			QString dir = locationHint();
 		
-			if (dir != QString::null && load(path,dir,group,true))
+			if (dir != QString::null && loadFromFile(path,dir,group,true))
 				loadingFinished(url,true,false);
 			else
 				loadingFinished(url,false,true);
@@ -509,15 +496,11 @@ namespace kt
 	{
 		QString dir;
 		if (savedir.isEmpty() || !bt::Exists(savedir))
-		{
-			dir = Settings::saveDir().path();
-			if (!Settings::useSaveDir()  || dir.isNull())
-				dir =  QDir::homePath();
-		}
+			dir = locationHint();
 		else
 			dir = savedir;
 		
-		if (dir != QString::null && load(data,dir,group,false,url))
+		if (dir != QString::null && loadFromData(data,dir,group,false,url))
 			loadingFinished(url,true,false);
 		else
 			loadingFinished(url,false,true);
@@ -527,57 +510,15 @@ namespace kt
 	{
 		QString dir;
 		if (savedir.isEmpty() || !bt::Exists(savedir))
-		{
-			dir = Settings::saveDir().path();
-			if (!Settings::useSaveDir())
-			{
-				Out(SYS_GEN|LOG_NOTICE) << "Cannot load " << url.prettyUrl() << " silently, default save location not set !" << endl;
-				Out(SYS_GEN|LOG_NOTICE) << "Using home directory instead !" << endl;
-				dir = QDir::homePath();
-			}
-		}
+			dir = locationHint();
 		else
 			dir = savedir;
 		
-		if (dir != QString::null && load(data,dir,group,true,url))
+		if (dir != QString::null && loadFromData(data,dir,group,true,url))
 			loadingFinished(url,true,false);
 		else
 			loadingFinished(url,false,true);
 	}
-
-	/*
-	void Core::loadSilentlyDir(const KUrl& url, const KUrl& savedir)
-	{
-		if (url.isLocalFile())
-		{
-			QString path = url.path(); 
-			QString dir = savedir.path();
-			QFileInfo fi(dir);
-			if (!fi.exists() || !fi.isWritable() || !fi.isDir())
-			{
-				Out(SYS_GEN|LOG_NOTICE) << "Cannot load " << path << " silently, destination directory is not OK ! Using default save directory." << endl;
-				dir = Settings::saveDir().path();
-				if (!Settings::useSaveDir())
-				{
-					Out(SYS_GEN|LOG_NOTICE) << "Default save directory not set, using home directory !" << endl;
-					dir = QDir::homePath();
-				}
-			}
-			
-			if (dir != QString::null && load(path,dir,QString(),true))
-				loadingFinished(url,true,false);
-			else
-				loadingFinished(url,false,true);
-		}
-		else
-		{
-			// download to a random file in tmp
-			KIO::Job* j = KIO::storedGet(url);
-			custom_save_locations.insert(j,savedir); // keep track of save location
-			connect(j,SIGNAL(result(KJob*)),this,SLOT(downloadFinishedSilently( KJob* )));
-		}
-	}
-*/
 	
 	void Core::start(bt::TorrentInterface* tc)
 	{
@@ -603,9 +544,14 @@ namespace kt
 		}
 	}
 
-	void Core::stop(bt::TorrentInterface* tc, bool user)
+	void Core::stop(bt::TorrentInterface* tc)
 	{
-		qman->stop(tc, user);
+		qman->stop(tc);
+	}
+	
+	void Core::stop(QList<bt::TorrentInterface*> & todo)
+	{
+		qman->stop(todo);
 	}
 
 	QString Core::findNewTorrentDir() const
@@ -638,12 +584,11 @@ namespace kt
 		try
 		{
 			tc = new TorrentControl();
-			tc->init(qman,idir + "torrent",idir,QString::null,
-				 Settings::useSaveDir() ? Settings::saveDir().path() : QString());
+			tc->init(qman,idir + "torrent",idir,QString::null);
 				
 			qman->append(tc);
 			connectSignals(tc);
-			if (tc->getStats().autostart && tc->getStats().user_controlled && !tc->overMaxRatio() && !tc->overMaxSeedTime())
+			if (tc->getStats().autostart && !bt::QueueManagerInterface::enabled() && !tc->overMaxRatio() && !tc->overMaxSeedTime())
 				start(tc);
 			torrentAdded(tc);
 		}
@@ -669,17 +614,29 @@ namespace kt
 			Out(SYS_GEN|LOG_NOTICE) << "Loading " << idir << endl;
 			loadExistingTorrent(idir);
 		}
-		qman->orderQueue();
+		if (!kt::QueueManager::enabled())
+			qman->startAutoStartTorrents();
+		else
+			qman->orderQueue();
 	}
 
 	void Core::remove(bt::TorrentInterface* tc,bool data_to)
 	{
 		try
 		{
+			if (tc->getJobQueue()->runningJobs())
+			{
+				// if there are running jobs, schedule delete when they finish
+				delayed_removal.insert(tc,data_to);
+				connect(tc,SIGNAL(runningJobsDone(bt::TorrentInterface*)),
+						this,SLOT(delayedRemove(bt::TorrentInterface*)));
+				return;
+			}
+			
 			const bt::TorrentStats & s = tc->getStats();
 			removed_bytes_up += s.session_bytes_uploaded;
 			removed_bytes_down += s.session_bytes_downloaded;
-			stop(tc,true);
+			stop(tc);
 
 			QString dir = tc->getTorDir();
 			
@@ -693,17 +650,31 @@ namespace kt
 				gui->errorMsg(e.toString());
 			}
 			
+			// cleanup potential data scans
+			QMap<bt::TorrentInterface*,ScanListener*>::iterator i = active_scans.find(tc);
+			if (i != active_scans.end())
+				closeScanListener(i.value());
+			
 			torrentRemoved(tc);
 			gman->torrentRemoved(tc);
 			qman->torrentRemoved(tc);
 			gui->updateActions();
 			bt::Delete(dir,false);
+			delayed_removal.remove(tc);
 		}
 		catch (Error & e)
 		{
 			gui->errorMsg(e.toString());
 		}
 	}
+	
+	void Core::delayedRemove(bt::TorrentInterface* tc)
+	{
+		if (!delayed_removal.contains(tc))
+			return;
+		
+		remove(tc,delayed_removal[tc]);
+	}
 
 	void Core::setMaxDownloads(int max)
 	{
@@ -761,14 +732,11 @@ namespace kt
 	{
 		try
 		{
-			update_timer.stop();
 			// do nothing if new and old dir are the same
 			if (KUrl(data_dir) == KUrl(new_dir) || data_dir == (new_dir + bt::DirSeparator()))
-			{
-				update_timer.start(CORE_UPDATE_INTERVAL);
 				return true;
-			}
 
+			update_timer.stop();
 			// safety check
 			if (!bt::Exists(new_dir))
 				bt::MakeDir(new_dir);
@@ -832,15 +800,15 @@ namespace kt
 		update_timer.start(CORE_UPDATE_INTERVAL);
 	}
 
-	void Core::startAll(int type)
+	void Core::startAll()
 	{
-		qman->startall(type);
+		qman->startAll();
 		startUpdateTimer();
 	}
 
-	void Core::stopAll(int type)
+	void Core::stopAll()
 	{
-		qman->stopall(type);
+		qman->stopAll();
 	}
 	
 	void Core::startUpdateTimer()
@@ -876,7 +844,7 @@ namespace kt
 			while (i != qman->end())
 			{
 				bt::TorrentInterface* tc = *i;
-				if (tc->updateNeeded())
+				if (tc->getStats().running)
 				{
 					tc->update();
 					updated = true;
@@ -1028,11 +996,6 @@ namespace kt
 		return qman->getPausedState();
 	}
 
-	void Core::queue(bt::TorrentInterface* tc)
-	{
-		qman->queue(tc);
-	}
-
 	void Core::aboutToBeStarted(bt::TorrentInterface* tc,bool & ret)
 	{
 		ret = true;
@@ -1083,10 +1046,6 @@ namespace kt
 						ret = false;
 					}
 					break;
-				case MissingFilesDlg::QUIT:
-					ret = false;
-					QTimer::singleShot(500,kapp,SLOT(quit()));
-					break;
 				case MissingFilesDlg::NEW_LOCATION_SELECTED:
 					ret = true;
 					break;
@@ -1116,10 +1075,6 @@ namespace kt
 						ret = false;
 					}
 					break;
-				case MissingFilesDlg::QUIT:
-					ret = false;
-					QTimer::singleShot(500,kapp,SLOT(quit()));
-					break;
 				case MissingFilesDlg::DO_NOT_DOWNLOAD:
 					ret = false;
 					break;
@@ -1170,17 +1125,36 @@ namespace kt
 		Out(SYS_GEN|LOG_IMPORTANT) << "Doing an automatic data check on " 
 					<< tc->getStats().torrent_name << endl;
 		
-		gui->dataScan(tc,false,true,QString::null);
+		doDataCheck(tc);
 	}
 
 	void Core::doDataCheck(bt::TorrentInterface* tc)
 	{
-		bool dummy = false;
-		if (tc->isCheckingData(dummy))
-			return;
-		
-		gui->dataScan(tc,false,false,i18n("Checking Data Integrity"));
+		QMap<bt::TorrentInterface*,ScanListener*>::iterator itr = active_scans.find(tc);
+		if (itr == active_scans.end())
+		{
+			ScanListener* listener = new ScanListener(tc);
+			connect(listener,SIGNAL(closeRequested(ScanListener*)),this,SLOT(closeScanListener(ScanListener*)));
+			active_scans.insert(tc,listener);
+			gui->dataScanStarted(listener);
+			tc->startDataCheck(listener);
+		}
+		else
+		{
+			ScanListener* listener = itr.value();
+			if (listener->isFinished())
+				listener->restart();
+		}
 	}
+	
+	
+	void Core::closeScanListener(ScanListener* sl)
+	{
+		gui->dataScanClosed(sl);
+		active_scans.remove(sl->torrent());
+		sl->deleteLater();
+	}
+
 
 	void Core::onLowDiskSpace(bt::TorrentInterface * tc, bool stopped)
 	{
@@ -1201,7 +1175,7 @@ namespace kt
 		Uint32 num = mig.findTorrentsToBeMigrated();
 		if (num > 0)
 		{
-			if (KMessageBox::questionYesNo(gui,i18n("KTorrent has found %1 torrents from the KDE3 version of KTorrent, do you want to import them ?",num)) == KMessageBox::Yes)
+			if (KMessageBox::questionYesNo(gui,i18np("KTorrent has found a torrent from the KDE3 version of KTorrent, do you want to import it?", "KTorrent has found %1 torrents from the KDE3 version of KTorrent, do you want to import them?",num)) == KMessageBox::Yes)
 			{
 				mig.migrateFoundTorrents(qman);
 				foreach (const QString & s,mig.getSuccessFullImports())
@@ -1223,7 +1197,7 @@ namespace kt
 		}
 		else
 		{
-			KMessageBox::information(gui,i18n("No torrents from the KDE3 version were found !"));
+			KMessageBox::information(gui,i18n("No torrents from the KDE3 version were found."));
 		}
 	}
 	
@@ -1237,6 +1211,21 @@ namespace kt
 		Q_UNUSED(tc);
 		gui->updateActions();
 	}
+	
+	QString Core::locationHint() const
+	{
+		QString dir;
+		if (Settings::useSaveDir())
+			dir = Settings::saveDir().toLocalFile();
+		else
+			dir = Settings::lastSaveDir();
+		
+		
+		if (dir.isEmpty() || !QDir(dir).exists())
+			dir = QDir::homePath();
+		
+		return dir;
+	}
 }
 
 #include "core.moc"
