@@ -38,6 +38,7 @@ namespace bt
 
 namespace kt
 {
+	class ScanListener;
 	class GUI;
 	class PluginManager;
 	class GroupManager;
@@ -55,8 +56,8 @@ namespace kt
 		// implemented from CoreInterface
 		virtual void setKeepSeeding(bool ks);
 		virtual bool changeDataDir(const QString & new_dir);
-		virtual void startAll(int type);
-		virtual void stopAll(int type);
+		virtual void startAll();
+		virtual void stopAll();
 		virtual CurrentStats getStats();
 		virtual bool changePort(bt::Uint16 port);
 		virtual bt::Uint32 getNumTorrentsRunning() const;
@@ -132,7 +133,7 @@ namespace kt
 		 * @param group Group to add torrent to
 		 * @param silently Whether or not to do this silently
 		 */
-		bool load(const QString & file,const QString & dir,const QString & group,bool silently);
+		bool loadFromFile(const QString & file,const QString & dir,const QString & group,bool silently);
 		
 		/**
 		 * Load a torrent file. Pops up an error dialog
@@ -141,47 +142,18 @@ namespace kt
 		 * @param dir Directory to save the data
 		 * @param silently Whether or not to do this silently
 		 */
-		bool load(const QByteArray & data,const QString & dir,const QString & group,bool silently, const KUrl& url);
+		bool loadFromData(const QByteArray & data,const QString & dir,const QString & group,bool silently, const KUrl& url);
 		
-		/**
-		 * Remove a download.This will delete all temp
-		 * data from this TorrentInterface And delete the
-		 * TorrentInterface itself. It can also potentially
-		 * start a new download (when one is waiting to be downloaded).
-		 * @param tc
-		 * @param data_to 
-		 */
-		virtual void remove(bt::TorrentInterface* tc,bool data_to);
-
 		/**
 		 * Update all torrents.
 		 */
 		void update();
 		
-		/**
-		 * Start a torrent, takes into account the maximum number of downloads.
-		 * @param tc The TorrentInterface
-		 */
 		virtual void start(bt::TorrentInterface* tc);
-
-		/**
-		 * Start a list of torrents
-		 * @param todo The list of torrents
-		 */
 		virtual void start(QList<bt::TorrentInterface*> & todo);
-		
-		/**
-		 * Stop a torrent, may start another download if it hasn't been started.
-		 * @param tc The TorrentInterface
-		 */
-		virtual void stop(bt::TorrentInterface* tc, bool user = false);
-		
-		/**
-		 * Enqueue/Dequeue function. Places a torrent in queue. 
-		 * If the torrent is already in queue this will remove it from queue.
-		 * @param tc TorrentControl pointer.
-		 */
-		virtual void queue(bt::TorrentInterface* tc);
+		virtual void stop(bt::TorrentInterface* tc);
+		virtual void stop(QList<bt::TorrentInterface*> & todo);
+		virtual void remove(bt::TorrentInterface* tc,bool data_to);
 		
 		/**
 		 * A torrent is about to be started. We will do some file checks upon this signal.
@@ -270,7 +242,8 @@ namespace kt
 	private:
 		void rollback(const QList<bt::TorrentInterface*> & success);
 		void connectSignals(bt::TorrentInterface* tc);
-		bool init(bt::TorrentControl* tc,const QString & group,bool silently);
+		bool init(bt::TorrentControl* tc,const QString & group,const QString & location,bool silently);
+		QString locationHint() const;
 
 	public:
 		void loadTorrents();
@@ -284,6 +257,8 @@ namespace kt
 		void emitCorruptedData(bt::TorrentInterface* tc);
 		void autoCheckData(bt::TorrentInterface* tc);
 		void checkForKDE3Torrents();
+		void closeScanListener(ScanListener* sl);
+		void delayedRemove(bt::TorrentInterface* tc);
 
 	private:
 		GUI* gui;
@@ -297,6 +272,8 @@ namespace kt
 		QMap<KJob*,KUrl> custom_save_locations; // map to store save locations
 		QMap<KJob*,QString> add_to_groups; // Map to keep track of which group to add a torrent to
 		int sleep_suppression_cookie;
+		QMap<bt::TorrentInterface*,ScanListener*> active_scans;
+		QMap<bt::TorrentInterface*,bool> delayed_removal;
 	};
 }
 
