@@ -31,20 +31,26 @@
 #include <groups/group.h>
 #include <util/log.h>
 #include <groups/groupmanager.h>
+#include <torrent/jobqueue.h>
 #include "gui.h"
 #include "view.h"
+#include "viewmodel.h"
 #include "viewmanager.h"
 #include "core.h"
 #include "groupview.h"
 #include "settings.h"
+#include "torrentactivity.h"
+#include "speedlimitsdlg.h"
+#include <kfiledialog.h>
+#include <kio/job.h>
 
 using namespace bt;
 
 namespace kt
 {
-	ViewManager::ViewManager(Group* all_group,GUI* gui,Core* core) : QObject(gui),gui(gui),core(core),current(0),all_group(all_group)
+	ViewManager::ViewManager(Group* all_group,GUI* gui,Core* core,TorrentActivity* ta) 
+		: QObject(ta),gui(gui),core(core),current(0),all_group(all_group),ta(ta)
 	{
-		view_menu = 0;
 	}
 
 	ViewManager::~ViewManager()
@@ -59,7 +65,7 @@ namespace kt
 		connect(v,SIGNAL(currentTorrentChanged(View* ,bt::TorrentInterface* )),
 			this,SLOT(onCurrentTorrentChanged(View* ,bt::TorrentInterface* )));
 		connect(v,SIGNAL(torrentSelectionChanged(View*)),
-				this,SLOT(onSelectionChaged(View*)));
+				this,SLOT(onSelectionChanged(View*)));
 		connect(v,SIGNAL(showMenu(View*, const QPoint&)),this,SLOT(showViewMenu(View*, const QPoint&)));
 		return v;
 	}
@@ -88,10 +94,10 @@ namespace kt
 		cv = g.readEntry("current_views",cv);
 
 		foreach (const QString &s,cv)
-			gui->openView(s,true);
+			ta->openView(s,true);
 
 		if (views.count() == 0)
-			gui->openNewView(all_group);
+			ta->openNewView(all_group);
 
 		int idx = 0;
 		foreach (View* v,views)
@@ -205,13 +211,6 @@ namespace kt
 		if (current)
 			current->checkData();
 	}
-		
-
-	void ViewManager::queueTorrents()
-	{
-		if (current)
-			current->queueTorrents();
-	}
 
 	void ViewManager::update()
 	{
@@ -238,6 +237,7 @@ namespace kt
 		
 	void ViewManager::onCurrentTabChanged(QWidget* tab)
 	{
+		current = 0;
 		foreach (View* v,views)
 		{
 			if (v == tab)
@@ -250,22 +250,16 @@ namespace kt
 		}
 	}
 
-	bool ViewManager::closeAllowed(QWidget* )
-	{
-		return views.count() > 1;
-	}
-
-	void ViewManager::tabCloseRequest(kt::GUIInterface* gui,QWidget* tab)
+	void ViewManager::removeView(View* view)
 	{
 		if (views.count() <= 1)
 			return;
 
 		foreach (View* v,views)
 		{
-			if (v == tab)
+			if (v == view)
 			{
 				views.removeAll(v);
-				gui->removeTabPage(v);				
 				v->deleteLater();
 				break;
 			}
@@ -277,9 +271,7 @@ namespace kt
 		if (current && current->getGroup() != g)
 		{
 			current->setGroup(g);
-			gui->changeTabIcon(current,g->groupIconName());
-			gui->changeTabText(current,current->caption(false));
-			gui->changeTabToolTip(current,current->caption(true));
+			ta->setTabProperties(current,current->caption(false),g->groupIconName(),current->caption(true));
 		}
 	}
 
@@ -289,13 +281,11 @@ namespace kt
 		{
 			if (v->getGroup() == g)
 			{
-				gui->changeTabIcon(v,g->groupIconName());
-				gui->changeTabText(v,v->caption(false));
-				gui->changeTabToolTip(v,v->caption(true));
+				ta->setTabProperties(v,v->caption(false),g->groupIconName(),v->caption(true));
 			}
 		}
 		
-		QMap<Group*,QAction*>::iterator j = group_actions.find(g);
+		QMap<Group*,KAction*>::iterator j = group_actions.find(g);
 		if (j != group_actions.end())
 			j.value()->setText(g->groupName());
 	}
@@ -311,7 +301,7 @@ namespace kt
 				if (views.count() > 1)
 				{
 					// remove the view 
-					gui->removeTabPage(v);
+					ta->removeView(v);
 					i = views.erase(i);
 					v->deleteLater();
 					if (current == v)
@@ -321,9 +311,7 @@ namespace kt
 				{
 					// change the current view to the all group
 					v->setGroup(all_group);
-					gui->changeTabIcon(v,all_group->groupIconName());
-					gui->changeTabText(v,v->caption(false));
-					gui->changeTabToolTip(v,v->caption(true));
+					ta->setTabProperties(v,v->caption(false),g->groupIconName(),v->caption(true));
 					i++;
 				}
 			}
@@ -332,13 +320,23 @@ namespace kt
 		}
 		
 		gui->unplugActionList("view_groups_list");
-		QMap<Group*,QAction*>::iterator j = group_actions.find(g);
-		if (j != group_actions.end())
+		QList<QAction*> actions;
+		QMap<Group*,KAction*>::iterator j = group_actions.begin();
+		while (j != group_actions.end())
 		{
-			delete j.value();
-			group_actions.erase(j);
+			if (j.key() == g)
+			{
+				delete j.value();
+				j = group_actions.erase(j);
+			}
+			else
+			{
+				actions.append(j.value());
+				j++;
+			}
 		}
-		gui->plugActionList("view_groups_list",group_actions.values());
+		
+		gui->plugActionList("view_groups_list",actions);
 	}
 	
 	void ViewManager::onGroupAdded(kt::Group* g)
@@ -347,16 +345,25 @@ namespace kt
 		KAction* act = new KAction(KIcon("application-x-bittorrent"),g->groupName(),this);
 		connect(act,SIGNAL(triggered()),this,SLOT(addToGroupItemTriggered()));
 		group_actions.insert(g,act);
-		gui->plugActionList("view_groups_list",group_actions.values());
+		
+		QList<QAction*> actions;
+		QMap<Group*,KAction*>::iterator j = group_actions.begin();
+		while (j != group_actions.end())
+		{
+			actions.append(j.value());
+			j++;
+		}
+		
+		gui->plugActionList("view_groups_list",actions);
 	}
 
 	void ViewManager::onCurrentTorrentChanged(View* v,bt::TorrentInterface* tc)
 	{
 		if (v == current)
-			gui->currentTorrentChanged(tc);
+			ta->currentTorrentChanged(tc);
 	}
 
-	void ViewManager::onSelectionChaged(View* v)
+	void ViewManager::onSelectionChanged(View* v)
 	{
 		if (v != current)
 			return;
@@ -364,6 +371,19 @@ namespace kt
 		updateActions();
 	}
 	
+	
+	void ViewManager::dataScanStarted(ScanListener* listener)
+	{
+		foreach (View* v,views)
+			v->dataScanStarted(listener);
+	}
+	
+	void ViewManager::dataScanClosed(ScanListener* listener)
+	{
+		foreach (View* v,views)
+			v->dataScanClosed(listener);
+	}
+	
 	void ViewManager::updateActions()
 	{
 		if (!current)
@@ -372,6 +392,7 @@ namespace kt
 		QList<bt::TorrentInterface*> sel;
 		current->getSelection(sel);		
 		
+		bool qm_enabled = !Settings::manuallyControlTorrents();
 		bool en_start = false;
 		bool en_stop = false;
 		bool en_remove = false;
@@ -379,7 +400,6 @@ namespace kt
 		bool en_announce = false;
 		bool en_add_peer = false;
 		bool en_peer_sources = false;
-		bool dummy = false;
 
 		foreach (bt::TorrentInterface* tc,sel)
 		{
@@ -388,27 +408,33 @@ namespace kt
 			if (tc->readyForPreview() && !s.multi_file_torrent)
 				en_prev = true;
 			
-			if (!tc->isCheckingData(dummy))
-				en_remove = true;
+			if (tc->getJobQueue()->runningJobs())
+				continue;
 			
+			en_remove = true;
 			if (!s.running)
-			{
-				if (!tc->isCheckingData(dummy))
+			{	
+				if (qm_enabled)
+				{
+					// Queued torrents can be stopped, and not started
+					if (s.queued)
+						en_stop = true;
+					else
+						en_start = true;
+				}
+				else
 				{
 					en_start = true;
 				}
 			}
 			else
 			{
-				if (!tc->isCheckingData(dummy))
-				{
-					en_stop = true;
-					if (tc->announceAllowed())
-						en_announce = true;
-				}
+				en_stop = true;
+				if (tc->announceAllowed())
+					en_announce = true;
 			}
 			
-			if (!s.priv_torrent && !tc->isCheckingData(dummy))
+			if (!s.priv_torrent)
 			{
 				en_add_peer = true;
 				en_peer_sources = true;
@@ -426,7 +452,6 @@ namespace kt
 		manual_announce->setEnabled(en_announce);
 		do_scrape->setEnabled(sel.count() > 0);
 		move_data->setEnabled(sel.count() > 0);
-		queue_torrent->setEnabled(en_remove);
 
 		const kt::Group* current_group = current->getGroup();
 		remove_from_group->setEnabled(current_group && !current_group->isStandardGroup());
@@ -440,7 +465,7 @@ namespace kt
 			
 			TorrentInterface* tc = sel.front();
 			// no data check when we are preallocating diskspace
-			check_data->setEnabled(tc->getStats().status != bt::ALLOCATING_DISKSPACE && !tc->isCheckingData(dummy));
+			check_data->setEnabled(tc->getStats().status != bt::ALLOCATING_DISKSPACE);
 			
 			if (en_peer_sources)
 			{
@@ -461,17 +486,41 @@ namespace kt
 		open_dir_menu->setEnabled(sel.count() == 1);
 		add_to_new_group->setEnabled(sel.count() > 0);
 		copy_url->setEnabled(sel.count() == 1 && sel.front()->loadUrl().isValid());
+		export_torrent->setEnabled(sel.count() == 1);
 		
-		start_all->setEnabled(current->numRunningTorrents() < current->numTorrents());
-		stop_all->setEnabled(current->numRunningTorrents() > 0);
+		if (qm_enabled)
+		{
+			QList<bt::TorrentInterface*> all;
+			current->viewModel()->allTorrents(all);
+			start_all->setEnabled(false);
+			stop_all->setEnabled(false);
+			foreach (bt::TorrentInterface* tc,all)
+			{
+				if (tc->getJobQueue()->runningJobs())
+					continue;
+				
+				const TorrentStats & s = tc->getStats();
+				if (s.running || (tc->isAllowedToStart() && !tc->overMaxRatio() && !tc->overMaxSeedTime()))
+					stop_all->setEnabled(true);
+				else
+					start_all->setEnabled(true);
+				
+				if (stop_all->isEnabled() && start_all->isEnabled())
+					break;
+			}
+		}
+		else
+		{
+			start_all->setEnabled(current->numRunningTorrents() < current->numTorrents());
+			stop_all->setEnabled(current->numRunningTorrents() > 0);
+		}
 		
 		// check for all views if the caption needs to be updated
 		foreach (View* v,views)
 		{
 			if (v->needToUpdateCaption())
 			{
-				gui->changeTabText(v,v->caption(false));
-				gui->changeTabToolTip(v,v->caption(true));
+				ta->setTabProperties(v,v->caption(false),v->getGroup()->groupIconName(),v->caption(true));
 			}
 		}
 	}
@@ -479,12 +528,37 @@ namespace kt
 	void ViewManager::setupActions()
 	{
 		KActionCollection* ac = gui->actionCollection();
+		KStandardAction::selectAll(this,SLOT(selectAll()),ac);
 		
-		start_torrent = ac->action("start");
-		stop_torrent = ac->action("stop");
-		remove_torrent = ac->action("remove");
-		start_all = ac->action("start_all");
-		stop_all = ac->action("stop_all");
+		start_torrent = new KAction(KIcon("kt-start"),i18n("Start"), this);
+		start_torrent->setToolTip(i18n("Start all selected torrents in the current tab"));
+		start_torrent->setShortcut(KShortcut(Qt::CTRL + Qt::Key_S));
+		connect(start_torrent,SIGNAL(triggered()),this,SLOT(startTorrents()));
+		ac->addAction("start",start_torrent);
+		
+		stop_torrent = new KAction(KIcon("kt-stop"),i18n("Stop"),this);
+		stop_torrent->setToolTip(i18n("Stop all selected torrents in the current tab"));
+		stop_torrent->setShortcut(KShortcut(Qt::CTRL + Qt::Key_H));
+		connect(stop_torrent,SIGNAL(triggered()),this,SLOT(stopTorrents()));
+		ac->addAction("stop",stop_torrent);
+		
+		remove_torrent = new KAction(KIcon("kt-remove"),i18n("Remove"),this);
+		remove_torrent->setToolTip(i18n("Remove all selected torrents in the current tab"));
+		remove_torrent->setShortcut(KShortcut(Qt::Key_Delete));
+		connect(remove_torrent,SIGNAL(triggered()),this,SLOT(removeTorrents()));
+		ac->addAction("remove",remove_torrent);
+		
+		start_all = new KAction(KIcon("kt-start-all"),i18n("Start All"),this);
+		start_all->setToolTip(i18n("Start all torrents in the current tab"));
+		start_all->setShortcut(KShortcut(Qt::SHIFT + Qt::Key_S));
+		connect(start_all,SIGNAL(triggered()),this,SLOT(startAllTorrents()));
+		ac->addAction("start_all",start_all);
+		
+		stop_all = new KAction(KIcon("kt-stop-all"),i18n("Stop All"),this);
+		stop_all->setToolTip(i18n("Stop all torrents in the current tab"));
+		stop_all->setShortcut(KShortcut(Qt::SHIFT + Qt::Key_H));
+		connect(stop_all,SIGNAL(triggered()),this,SLOT(stopAllTorrents()));
+		ac->addAction("stop_all",stop_all);
 		
 		remove_torrent_and_data = new KAction(KIcon("kt-remove"),i18n("Remove Torrent and Data"),this);
 		remove_torrent_and_data->setShortcut(KShortcut(Qt::CTRL + Qt::Key_Delete));
@@ -495,8 +569,6 @@ namespace kt
 		connect(rename_torrent,SIGNAL(triggered()),this,SLOT(renameTorrent()));
 		ac->addAction("view_rename_torrent",rename_torrent);
 		
-		queue_torrent = ac->action("queue_action");
-		
 		add_peers = new KAction(KIcon("list-add"),i18n("Add Peers"),this);
 		connect(add_peers,SIGNAL(triggered()),this,SLOT(addPeers()));
 		ac->addAction("view_add_peers",add_peers);
@@ -546,7 +618,11 @@ namespace kt
 		connect(add_to_new_group,SIGNAL(triggered()),this,SLOT(addToNewGroup()));
 		ac->addAction("view_add_to_new_group",add_to_new_group);
 		
-		check_data = ac->action("check_data");
+		check_data = new KAction(KIcon("kt-check-data"),i18n("Check Data"),this);
+		check_data->setToolTip(i18n("Check all the data of a torrent"));
+		check_data->setShortcut(KShortcut(Qt::SHIFT + Qt::Key_C));
+		connect(check_data,SIGNAL(triggered()),this,SLOT(checkData()));
+		ac->addAction("check_data",check_data);
 		
 		GroupManager* gman = core->getGroupManager();
 		for (GroupManager::iterator i = gman->begin();i != gman->end();i++)
@@ -565,6 +641,16 @@ namespace kt
 		copy_url = new KAction(KIcon("edit-copy"),i18n("Copy Torrent URL"),this);
 		connect(copy_url,SIGNAL(triggered()),this,SLOT(copyTorrentURL()));
 		ac->addAction("view_copy_url",copy_url);
+		
+		export_torrent = new KAction(KIcon("document-export"),i18n("Export Torrent"),this);
+		connect(export_torrent,SIGNAL(triggered()),this,SLOT(exportTorrent()));
+		ac->addAction("view_export_torrent",export_torrent);
+		
+		speed_limits = new KAction(KIcon("kt-speed-limits"),i18n("Speed Limits"),this);
+		speed_limits->setToolTip(i18n("Set the speed limits of individual torrents"));
+		connect(speed_limits,SIGNAL(triggered()),this,SLOT(speedLimits()));
+		speed_limits->setShortcut(KShortcut(Qt::CTRL + Qt::Key_L));
+		ac->addAction("speed_limits",speed_limits);
 	}
 	
 	void ViewManager::addToGroupItemTriggered()
@@ -572,9 +658,9 @@ namespace kt
 		if (!current)
 			return;
 		
-		QAction* s = (QAction*)sender();
+		KAction* s = (KAction*)sender();
 		Group* g = 0;
-		QMap<Group*,QAction*>::iterator j = group_actions.begin();
+		QMap<Group*,KAction*>::iterator j = group_actions.begin();
 		while (j != group_actions.end() && !g)
 		{
 			if (j.value() == s)
@@ -597,7 +683,7 @@ namespace kt
 	
 	void ViewManager::addToNewGroup()
 	{
-		GroupView* gv = gui->getGroupView();
+		GroupView* gv = ta->getGroupView();
 		Group* g = gv->addNewGroup();
 		if (g && current)
 		{
@@ -635,23 +721,46 @@ namespace kt
 			current->selectAll();
 	}
 	
+	void ViewManager::speedLimits()
+	{
+		QList<bt::TorrentInterface*> sel;
+		getSelection(sel);
+		SpeedLimitsDlg dlg(sel.count() > 0 ? sel.front() : 0,core,gui->getMainWindow());
+		dlg.exec();
+	}
+	
+	void ViewManager::exportTorrent()
+	{
+		QList<bt::TorrentInterface*> sel;
+		getSelection(sel);
+		if (sel.count() == 1)
+		{
+			bt::TorrentInterface* tc = sel.front();
+			QString filter = "*.torrent|" + i18n("Torrents (*.torrent)");
+			QString fn = KFileDialog::getSaveFileName(KUrl("kfiledialog:///exportTorrent"),filter);
+			if (!fn.isEmpty())
+				KIO::file_copy(tc->getTorDir() + "torrent",fn);
+		}
+	}
+	
 	void ViewManager::showViewMenu(View* v,const QPoint & pos)
 	{
 		if (!v)
 			return;
 		
+		KMenu* view_menu = qobject_cast<KMenu*>(gui->container("ViewMenu"));
 		if (!view_menu)
-		{
-			view_menu = (KMenu*)gui->container("ViewMenu");
-			if (!view_menu)
-			{
-				Out(SYS_GEN|LOG_NOTICE) << "Failed to create ViewMenu" << endl;
-				return;
-			}
-			gui->plugActionList("view_groups_list",group_actions.values());
-		}
+			return;
 		
-	
+		QList<QAction*> actions;
+		QMap<Group*,KAction*>::iterator j = group_actions.begin();
+		while (j != group_actions.end())
+		{
+			actions.append(j.value());
+			j++;
+		}
+			
+		gui->plugActionList("view_groups_list",actions);
 		
 		gui->unplugActionList("view_columns_list");
 		gui->plugActionList("view_columns_list",v->columnActionList());
