@@ -33,7 +33,10 @@
 #include <groups/group.h>
 #include "viewmodel.h"
 #include "core.h"
+#include <interfaces/trackerinterface.h>
 #include <util/sha1hash.h>
+#include "viewdelegate.h"
+#include "view.h"
 
 using namespace bt;
 
@@ -56,7 +59,7 @@ namespace kt
 		leechers_total = s.leechers_total;
 		leechers_connected_to = s.leechers_connected_to;
 		percentage = Percentage(s);
-		share_ratio = ShareRatio(s);
+		share_ratio = s.shareRatio();
 		runtime_dl = tc->getRunningTimeDL();
 		runtime_ul = tc->getRunningTimeUL() - tc->getRunningTimeDL();
 		hidden = false;
@@ -151,7 +154,7 @@ namespace kt
 				ret = true;
 		}
 		
-		float ratio = ShareRatio(s);
+		float ratio = s.shareRatio();
 		if (fabsf(share_ratio - ratio) > 0.01)
 		{
 			modified = true;
@@ -187,7 +190,7 @@ namespace kt
 		switch (col)
 		{
 			case 0: return tc->getDisplayName();
-			case 1: return tc->statusToString();
+			case 1: return s.statusToString();
 			case 2: return BytesToString(bytes_downloaded);
 			case 3: return BytesToString(total_bytes_to_download);
 			case 4: return BytesToString(bytes_uploaded);
@@ -214,7 +217,7 @@ namespace kt
 			case 8: return QString("%1 (%2)").arg(QString::number(seeders_connected_to)).arg(QString::number(seeders_total));
 			case 9: return QString("%1 (%2)").arg(QString::number(leechers_connected_to)).arg(QString::number(leechers_total));
 			// xgettext: no-c-format
-			case 10: return i18n("%1 %",KGlobal::locale()->formatNumber(percentage,2));
+			case 10: return percentage;
 			case 11: return KGlobal::locale()->formatNumber(share_ratio,2);
 			case 12: return DurationToString(runtime_dl);
 			case 13: return DurationToString(runtime_ul);
@@ -228,7 +231,7 @@ namespace kt
 		switch (col)
 		{
 			case 0: return QString::localeAwareCompare(tc->getDisplayName(),other->tc->getDisplayName()) < 0;
-			case 1: return tc->statusToString() < other->tc->statusToString();
+			case 1: return tc->getStats().statusToString() < other->tc->getStats().statusToString();
 			case 2: return bytes_downloaded < other->bytes_downloaded;
 			case 3: return total_bytes_to_download < other->total_bytes_to_download;
 			case 4: return bytes_uploaded < other->bytes_uploaded;
@@ -296,7 +299,7 @@ namespace kt
 			return group->isMember(tc);
 	}
 
-	ViewModel::ViewModel(Core* core,QObject* parent) : QAbstractTableModel(parent),core(core)
+	ViewModel::ViewModel(Core* core,View* parent) : QAbstractTableModel(parent),core(core),view(parent)
 	{
 		connect(core,SIGNAL(torrentAdded(bt::TorrentInterface*)),this,SLOT(addTorrent(bt::TorrentInterface*)));
 		connect(core,SIGNAL(torrentRemoved(bt::TorrentInterface*)),this,SLOT(removeTorrent(bt::TorrentInterface*)));
@@ -322,13 +325,12 @@ namespace kt
 	void ViewModel::setGroup(Group* g)
 	{
 		group = g;
-		update();
 	}
 	
 	void ViewModel::addTorrent(bt::TorrentInterface* ti)
 	{
 		torrents.append(new Item(ti));
-		update(true);
+		update(view->viewDelegate(),true);
 	}
 	
 	void ViewModel::removeTorrent(bt::TorrentInterface* ti)
@@ -340,7 +342,7 @@ namespace kt
 			if (item->tc == ti)
 			{
 				removeRow(idx);
-				update(true);
+				update(view->viewDelegate(),true);
 				break;
 			}
 			idx++;
@@ -354,7 +356,7 @@ namespace kt
 		//emit dataChanged(createIndex(row,0),createIndex(row,14));
 	}
 
-	bool ViewModel::update(bool force_resort)
+	bool ViewModel::update(ViewDelegate* delegate,bool force_resort)
 	{
 		bool resort = force_resort;
 		Uint32 idx=0;
@@ -365,11 +367,14 @@ namespace kt
 			if (i->update(sort_column,modified))
 				resort = true;
 			
-			bool hidden = !i->member(group);
+			bool hidden = !i->member(group)/* && !delegate->extended(i->tc) */;
 			if (hidden != i->hidden)
 			{
 				i->hidden = hidden;
 				resort = true;
+				// hide the extender if there is one shown
+				if (hidden && delegate->extended(i->tc))
+					delegate->hideExtender(i->tc);
 			}
 			
 			if (!i->hidden)
@@ -497,7 +502,8 @@ namespace kt
 		else if (role == Qt::DecorationRole && index.column() == 1)
 		{
 			bt::TorrentInterface* tc = item->tc;
-			if (tc->getStats().tracker_status == bt::TRACKER_ERROR)
+			bt::TrackerInterface* trk = tc->getTrackersList()->getCurrentTracker();
+			if (trk && trk->trackerStatus() == bt::TRACKER_ERROR)
 				return KIcon("dialog-warning");
 		} 
 		else if (role == Qt::ToolTipRole)
@@ -505,8 +511,9 @@ namespace kt
 			if (index.column() == 1)
 			{
 				bt::TorrentInterface* tc = item->tc;
-				if (tc->getStats().tracker_status == bt::TRACKER_ERROR)
-					return i18n("There is a problem with the tracker: <br><b>%1</b>",tc->getStats().tracker_status_string);
+				bt::TrackerInterface* trk = tc->getTrackersList()->getCurrentTracker();
+				if (trk && trk->trackerStatus() == bt::TRACKER_ERROR)
+					return trk->trackerStatusString();
 			}
 			else if (index.column() == 0)
 			{
@@ -543,9 +550,9 @@ namespace kt
 	Qt::ItemFlags ViewModel::flags(const QModelIndex & index) const
 	{
 		if (!index.isValid() || index.row() >= torrents.count() || index.row() < 0)
-			return QAbstractTableModel::flags(index);
+			return QAbstractTableModel::flags(index) | Qt::ItemIsDropEnabled;
 		
-		Qt::ItemFlags flags = QAbstractTableModel::flags(index) | Qt::ItemIsDragEnabled;
+		Qt::ItemFlags flags = QAbstractTableModel::flags(index) | Qt::ItemIsDragEnabled | Qt::ItemIsDropEnabled;
 		if (index.column() == 0 )
 			flags |= Qt::ItemIsEditable;
 		
@@ -556,6 +563,7 @@ namespace kt
 	{
 		QStringList types;
 		types << "application/x-ktorrent-drag-object";
+		types << "text/uri-list";
 		return types;
 	}
 	
@@ -588,6 +596,31 @@ namespace kt
 		mime_data->setData( "application/x-ktorrent-drag-object", encoded_data);
 		return mime_data;
 	}
+		
+	bool ViewModel::dropMimeData(const QMimeData* data, Qt::DropAction action, int row, int column, const QModelIndex& parent)
+	{
+		Q_UNUSED(row);
+		Q_UNUSED(column);
+		Q_UNUSED(parent);
+		if (action == Qt::IgnoreAction)
+			return true;
+		
+		if (!data->hasUrls())
+			return false;
+		
+		QList<QUrl> files = data->urls();
+		foreach (QUrl file,files)
+		{
+			core->load(file,QString());
+		}
+		
+		return true;
+	}
+	
+	Qt::DropActions ViewModel::supportedDropActions() const
+	{
+		return Qt::CopyAction | Qt::MoveAction;
+	}
 	
 	void ViewModel::torrentsFromIndexList(const QModelIndexList & idx,QList<bt::TorrentInterface*> & tlist)
 	{
