@@ -78,6 +78,7 @@ namespace bt
 		else
 			output_dir = this->datadir;
 		files.setAutoDelete(true);
+		dnd_files.setAutoDelete(true);
 	}
 
 
@@ -252,7 +253,7 @@ namespace bt
 		saveFileMap();
 	}
 	
-	KJob* MultiFileCache::moveDataFiles(const QString & ndir)
+	Job* MultiFileCache::moveDataFiles(const QString & ndir)
 	{
 		if ( !bt::Exists ( ndir ) )
 			bt::MakeDir ( ndir );
@@ -290,12 +291,11 @@ namespace bt
 		}
 		else
 		{
-			job->startMoving();
 			return job;
 		}
 	}
 	
-	void MultiFileCache::moveDataFilesFinished(KJob* job)
+	void MultiFileCache::moveDataFilesFinished(Job* job)
 	{
 		if (job->error())
 			return;
@@ -312,53 +312,16 @@ namespace bt
 		}
 	}
 	
-	KJob* MultiFileCache::moveDataFiles(const QMap<TorrentFileInterface*,QString> & files)
+	Job* MultiFileCache::moveDataFiles(const QMap<TorrentFileInterface*,QString> & files)
 	{
 		if (files.count() == 0)
 			return 0;
 		
-		MoveDataFilesJob* job = new MoveDataFilesJob();
-		int nmoves = 0;
-		QMap<TorrentFileInterface*,QString>::const_iterator i = files.begin();
-		while (i != files.end())
-		{
-			TorrentFileInterface* tf = i.key();
-			QString dest = i.value();
-			if (QFileInfo(dest).isDir())
-			{
-				QString path = tf->getUserModifiedPath();
-				if (!dest.endsWith(bt::DirSeparator()))
-					dest += bt::DirSeparator();
-			
-				int last = path.lastIndexOf(bt::DirSeparator());
-				QString dst = dest + path.mid(last+1);
-				if (QFileInfo(tf->getPathOnDisk()).canonicalPath() != QFileInfo(dst).canonicalPath())
-				{
-					job->addMove(tf->getPathOnDisk(),dst);
-					nmoves++;
-				}
-			}
-			else if (QFileInfo(tf->getPathOnDisk()).canonicalPath() != QFileInfo(i.value()).canonicalPath())
-			{
-				job->addMove(tf->getPathOnDisk(),i.value());
-				nmoves++;
-			}
-			i++;
-		}
-		
-		if (nmoves)
-		{
-			job->startMoving();
-			return job;
-		}
-		else
-		{
-			delete job;
-			return 0;
-		}
+		MoveDataFilesJob* job = new MoveDataFilesJob(files);
+		return job;
 	}
 		
-	void MultiFileCache::moveDataFilesFinished(const QMap<TorrentFileInterface*,QString> & fmap,KJob* job)
+	void MultiFileCache::moveDataFilesFinished(const QMap<TorrentFileInterface*,QString> & fmap,Job* job)
 	{
 		if (job->error())
 			return;
@@ -635,6 +598,13 @@ namespace bt
 		}
 		
 		Uint8* data = piece->data();
+		if (!data) // this should not happen but just in case
+		{
+			if (!piece->inUse())
+				clearPiece(piece);
+			return;
+		}
+		
 		Chunk* c = piece->parentChunk();
 		QList<Uint32> tflist;
 		tor.calcChunkPos(c->getIndex(),tflist);
@@ -768,15 +738,7 @@ namespace bt
 		if (!fptr.open(src_file,"rb"))
 			throw Error(i18n("Cannot open file %1 : %2",src_file,fptr.errorString()));
 		
-		Uint32 cs = 0;
-		if (tf->getFirstChunk() == tor.getNumChunks() - 1)
-		{
-			cs = tor.getFileLength() % tor.getChunkSize();
-			if (cs == 0)
-				cs = tor.getChunkSize();
-		}
-		else
-			cs = tor.getChunkSize();
+		Uint32 cs = (tf->getFirstChunk() == tor.getNumChunks() - 1) ? tor.getLastChunkSize() : tor.getChunkSize();
 		
 		Uint8* tmp = new Uint8[tor.getChunkSize()];
 		try
@@ -813,12 +775,12 @@ namespace bt
 		{
 			bool res = false;
 			
-			#ifdef HAVE_XFS_XFS_H
-				if( (! res) && Cache::useFSSpecificPreallocMethod() )
-				{
-					res = XfsPreallocate(output_file, tf->getSize()) );
-				}
-			#endif
+#ifdef HAVE_XFS_XFS_H
+			if (Cache::preallocateFully())
+			{
+				res = XfsPreallocate(output_file, tf->getSize()) );
+			}
+#endif
 			
 			if(! res)
 			{
@@ -834,15 +796,7 @@ namespace bt
 			}
 		}
 		
-		Uint32 cs = 0;
-		if (tf->getFirstChunk() == tor.getNumChunks() - 1)
-		{
-			cs = tor.getFileLength() % tor.getChunkSize();
-			if (cs == 0)
-				cs = tor.getChunkSize();
-		}
-		else
-			cs = tor.getChunkSize();
+		Uint32 cs = (tf->getFirstChunk() == tor.getNumChunks() - 1) ? tor.getLastChunkSize() : tor.getChunkSize();
 		
 		File fptr;
 		if (!fptr.open(output_file,"r+b"))
@@ -855,8 +809,12 @@ namespace bt
 		
 		try
 		{
-			dnd.readFirstChunk(tmp,0,cs - tf->getFirstChunkOffset());
-			fptr.write(tmp,cs - tf->getFirstChunkOffset());
+			Uint32 to_read = cs - tf->getFirstChunkOffset();
+			if (to_read > tf->getSize()) // check for files which are smaller then a chunk
+				to_read = tf->getSize();
+			
+			dnd.readFirstChunk(tmp,0,to_read);
+			fptr.write(tmp,to_read);
 			
 			if (tf->getFirstChunk() != tf->getLastChunk())
 			{
@@ -960,7 +918,7 @@ namespace bt
 		}
 	}
 	
-	KJob* MultiFileCache::deleteDataFiles()
+	Job* MultiFileCache::deleteDataFiles()
 	{
 		DeleteDataFilesJob* job = new DeleteDataFilesJob(output_dir);
 		for (Uint32 i = 0;i < tor.getNumFiles();i++)
@@ -977,7 +935,6 @@ namespace bt
 			job->addEmptyDirectoryCheck(tf.getUserModifiedPath());
 		}
 		
-		job->start();
 		return job;
 	}
 	
