@@ -30,6 +30,7 @@
 #include <interfaces/webseedinterface.h>
 #include <bcodec/bencoder.h>
 #include "dbustorrent.h"
+#include <interfaces/trackerinterface.h>
 
 using namespace bt;
 
@@ -152,23 +153,23 @@ namespace kt
 	
 	QString DBusTorrent::currentTracker() const
 	{
-		return ti->getTrackersList()->getTrackerURL().prettyUrl();
+		bt::TrackerInterface* t = ti->getTrackersList()->getCurrentTracker();
+		return t ? t->trackerURL().prettyUrl() : QString();
 	}
 	
 	QStringList DBusTorrent::trackers() const
 	{
-		TrackersList* tlist = ti->getTrackersList();
-		KUrl::List urls = tlist->getTrackerURLs();
+		QList<bt::TrackerInterface*> trackers = ti->getTrackersList()->getTrackers();
 		QStringList ret;
-		foreach (const KUrl& u,urls)
-			ret << u.prettyUrl();
+		foreach (bt::TrackerInterface* t,trackers)
+			ret << t->trackerURL().prettyUrl();
 		return ret;
 	}
 	
 	void DBusTorrent::changeTracker(const QString & tracker_url)
 	{
 		KUrl url(tracker_url);
-		ti->getTrackersList()->setTracker(url);
+		ti->getTrackersList()->setCurrentTracker(url);
 	}
 	
 	void DBusTorrent::announce()
@@ -272,20 +273,14 @@ namespace kt
 		enc.write("seeders_connected_to",s.seeders_connected_to);
 		enc.write("leechers_total",s.leechers_total);
 		enc.write("leechers_connected_to",s.leechers_connected_to);
-		enc.write("total_times_downloaded", s.total_times_downloaded);
-		enc.write("status",ti->statusToString());
-		enc.write("tracker_status", s.tracker_status_string);
+		enc.write("status",s.statusToString());
 		enc.write("session_bytes_downloaded", s.session_bytes_downloaded);
 		enc.write("session_bytes_uploaded", s.session_bytes_uploaded);
-		enc.write("trk_bytes_downloaded", s.trk_bytes_downloaded);
-		enc.write("trk_bytes_uploaded", s.trk_bytes_uploaded);
 		enc.write("output_path", s.output_path);
 		enc.write("running",s.running);
 		enc.write("started", s.started);
 		enc.write("multi_file_torrent", s.multi_file_torrent);
 		enc.write("stopped_by_error", s.stopped_by_error);
-		enc.write("completed", s.completed);
-		enc.write("user_controlled", s.user_controlled);
 		enc.write("max_share_ratio", s.max_share_ratio);
 		enc.write("max_seed_time", s.max_seed_time);
 		enc.write("num_corrupted_chunks", s.num_corrupted_chunks);
@@ -318,10 +313,10 @@ namespace kt
 		switch (reason)
 		{
 			case bt::MAX_RATIO_REACHED:
-				msg = i18n("Maximum share ratio reached !");
+				msg = i18n("Maximum share ratio reached.");
 				break;
 			case bt::MAX_SEED_TIME_REACHED:
-				msg = i18n("Maximum seed time reached !");
+				msg = i18n("Maximum seed time reached.");
 				break;
 		}
 		emit seedingAutoStopped(this,msg);
@@ -433,16 +428,17 @@ namespace kt
 	void DBusTorrent::setPriority(int p)
 	{
 		ti->setPriority(p);
-	}
+	}	
 	
-	bool DBusTorrent::isUserControlled() const
+	void DBusTorrent::setAllowedToStart(bool on)
 	{
-		return ti->isUserControlled();
+		ti->setAllowedToStart(on);
 	}
-	
-	void DBusTorrent::setUserControlled(bool uc)
+
+	bool DBusTorrent::isAllowedToStart() const
 	{
-		ti->setUserControlled(uc);
+		return ti->isAllowedToStart();
 	}
+
 }
 
