@@ -36,6 +36,8 @@ namespace kt
 	ScheduleItem::ScheduleItem() 
 		: day(0),upload_limit(0),download_limit(0),paused(false),set_conn_limits(false),global_conn_limit(0),torrent_conn_limit(0)
 	{
+		screensaver_limits = false;
+		ss_download_limit = ss_upload_limit = 0;
 	}
 	
 	ScheduleItem::ScheduleItem(const ScheduleItem & item)
@@ -71,6 +73,9 @@ namespace kt
 		upload_limit = item.upload_limit;
 		download_limit = item.download_limit;
 		paused = item.paused;
+		screensaver_limits = item.screensaver_limits;
+		ss_download_limit = item.ss_download_limit;
+		ss_upload_limit = item.ss_upload_limit;
 		set_conn_limits = item.set_conn_limits;
 		global_conn_limit = item.global_conn_limit;
 		torrent_conn_limit = item.torrent_conn_limit;
@@ -87,7 +92,16 @@ namespace kt
 				paused == item.paused && 
 				set_conn_limits == item.set_conn_limits &&
 				global_conn_limit == item.global_conn_limit &&
-				torrent_conn_limit == item.torrent_conn_limit;
+				torrent_conn_limit == item.torrent_conn_limit &&
+				screensaver_limits == item.screensaver_limits &&
+				ss_download_limit == item.ss_download_limit &&
+				ss_upload_limit == item.ss_upload_limit;
+	}
+	
+	void ScheduleItem::checkTimes()
+	{
+		start.setHMS(start.hour(),start.minute(),0);
+		end.setHMS(end.hour(),end.minute(),59);
 	}
 	
 	
@@ -123,13 +137,13 @@ namespace kt
 		{
 			delete node;
 			Out(SYS_SCD|LOG_NOTICE) << "Decoding " << file << " failed : " << err.toString() << endl;
-			throw bt::Error(i18n("The file %1 is corrupted or not a proper KTorrent schedule file !"));
+			throw bt::Error(i18n("The file %1 is corrupted or not a proper KTorrent schedule file."));
 		}
 		
 		if (!node)
 		{
 			Out(SYS_SCD|LOG_NOTICE) << "Decoding " << file << " failed !" << endl;
-			throw bt::Error(i18n("The file %1 is corrupted or not a proper KTorrent schedule file !"));
+			throw bt::Error(i18n("The file %1 is corrupted or not a proper KTorrent schedule file."));
 		}
 		
 		if (node->getType() == BNode::LIST)
@@ -184,6 +198,21 @@ namespace kt
 				item->set_conn_limits = true;
 			}
 		}
+		
+		BValueNode* ss_limits = dict->getValue(QString("screensaver_limits"));
+		if (ss_limits)
+		{
+			item->screensaver_limits = ss_limits->data().toInt() == 1;
+			item->ss_download_limit = dict->getInt("ss_download_limit");
+			item->ss_upload_limit = dict->getInt("ss_upload_limit");
+		}
+		else
+		{
+			item->screensaver_limits = false;
+			item->ss_download_limit = item->ss_upload_limit = 0;
+		}
+		
+		item->checkTimes();
 		return true;
 	}
 		
@@ -217,6 +246,9 @@ namespace kt
 				enc.write("per_torrent"); enc.write((Uint32)i->torrent_conn_limit);
 				enc.end();
 			}
+			enc.write("screensaver_limits",(Uint32)i->screensaver_limits);
+			enc.write("ss_upload_limit",i->ss_upload_limit);
+			enc.write("ss_download_limit",i->ss_download_limit);
 			enc.end();
 		}
 		enc.end();
@@ -256,7 +288,7 @@ namespace kt
 		ScheduleItem* item = getCurrentItem(now);
 		// when we are in the middle of a ScheduleItem, we need to trigger again at the end of it
 		if (item) 
-			return now.time().secsTo(item->end) + 1; // change the schedule one second after it expires
+			return now.time().secsTo(item->end) + 5; // change the schedule 5 seconds after it expires
 		
 		// lets look at all schedule items on the same day
 		// and find the next one
@@ -271,10 +303,10 @@ namespace kt
 		}
 		
 		if (item)
-			return now.time().secsTo(item->start);
+			return now.time().secsTo(item->start) + 5;
 		
 		QTime end_of_day(23,59,59);
-		return now.time().secsTo(end_of_day) + 1;
+		return now.time().secsTo(end_of_day) + 5;
 	}
 	
 	bool Schedule::modify(ScheduleItem* item,const QTime & start,const QTime & end,int day)
@@ -286,6 +318,7 @@ namespace kt
 		item->start = start;
 		item->end = end;
 		item->day = day;
+		item->checkTimes();
 		if (conflicts(item))
 		{
 			// restore old start and end time
