@@ -49,6 +49,8 @@
 #include "htmlpart.h"
 #include "searchplugin.h"
 #include "searchenginelist.h"
+#include "homepage.h"
+
 
 
 
@@ -57,7 +59,7 @@ using namespace bt;
 namespace kt
 {
 	
-	SearchWidget::SearchWidget(SearchPlugin* sp,SearchEngineList* sl) : html_part(0),sp(sp)
+	SearchWidget::SearchWidget(SearchPlugin* sp) : html_part(0),sp(sp)
 	{
 		QVBoxLayout* layout = new QVBoxLayout(this);
 		layout->setSpacing(0);
@@ -66,14 +68,16 @@ namespace kt
 		
 		KActionCollection* ac = sp->actionCollection();
 		sbar = new KToolBar(this);
+		sbar->setToolButtonStyle(Qt::ToolButtonIconOnly);
 		sbar->addAction(ac->action("search_tab_back"));
 		sbar->addAction(ac->action("search_tab_reload"));
+		sbar->addAction(ac->action("search_home"));
 		search_text = new KLineEdit(sbar);
 		sbar->addWidget(search_text);
 		sbar->addAction(ac->action("search_tab_search"));
 		sbar->addWidget(new QLabel(i18n(" Engine:")));
 		search_engine = new KComboBox(sbar);
-		search_engine->setModel(sl);
+		search_engine->setModel(sp->getSearchEngineList());
 		sbar->addWidget(search_engine);
 		
 		connect(search_text,SIGNAL(returnPressed()),this,SLOT(search()));;
@@ -103,9 +107,10 @@ namespace kt
 				this,SLOT(onOpenTorrent(const KUrl& )));
 		connect(html_part,SIGNAL(popupMenu(const QString&, const QPoint& )),
 				this,SLOT(showPopupMenu(const QString&, const QPoint& )));
-		connect(html_part,SIGNAL(searchFinished()),this,SLOT(onFinished()));
+		connect(html_part,SIGNAL(completed()),this,SLOT(onFinished()));
 		connect(html_part,SIGNAL(saveTorrent(const KUrl& )),
 				this,SLOT(onSaveTorrent(const KUrl& )));
+		connect(html_part,SIGNAL(searchRequested(QString)),this,SLOT(onSearchRequested(QString)));
 	
 		KParts::PartManager* pman = html_part->partManager();
 		connect(pman,SIGNAL(partAdded(KParts::Part*)),this,SLOT(onFrameAdded(KParts::Part* )));
@@ -167,8 +172,10 @@ namespace kt
 	{
 		if (html_part)
 		{
-			html_part->openUrl(url);
-			html_part->addToHistory(url);
+			if (url.protocol() == "home")
+				home();
+			else
+				html_part->openUrl(url);
 		}
 	
 		search_text->setText(sb_text);
@@ -186,13 +193,24 @@ namespace kt
 		if (search_engine->currentIndex() != engine)
 			search_engine->setCurrentIndex(engine);
 	
-		SearchEngineList & sl = sp->getSearchEngineList();
-		KUrl url = sl.search(engine,text);
+		KUrl url = sp->getSearchEngineList()->search(engine,text);
 	
 		statusBarMsg(i18n("Searching for %1...",text));
 		//html_part->openURL(url);
+		html_part->show();
  		html_part->openUrlRequest(url,KParts::OpenUrlArguments(),KParts::BrowserArguments());
-	}	
+	}
+	
+	void SearchWidget::onSearchRequested(const QString & text)
+	{
+		search(text,search_engine->currentIndex());
+	}
+
+	
+	void SearchWidget::setSearchBarEngine(int engine)
+	{
+		search_engine->setCurrentIndex(engine);
+	}
 	
 	void SearchWidget::onUrlHover(const QString & url)
 	{
@@ -201,6 +219,7 @@ namespace kt
 	
 	void SearchWidget::onFinished()
 	{
+		changeTitle(this,html_part->title());
 	}
 	
 	void SearchWidget::onOpenTorrent(const KUrl & url)
@@ -215,7 +234,7 @@ namespace kt
 		{
 			KUrl save_url = KUrl(fn);
 			// start a copy job
-			KIO::file_copy(url,save_url);
+			KIO::file_copy(url,save_url,-1,KIO::Overwrite);
 		}
 	}
 	
@@ -308,6 +327,12 @@ namespace kt
 		openNewTab(url_to_open);
 	}
 	
+	void SearchWidget::home() 
+	{
+		html_part->home();
+	}
+
+	
 	bool SearchWidget::backAvailable() const
 	{
 		return html_part->backAvailable();
