@@ -39,7 +39,6 @@ namespace kt
 	class HTMLPart;
 	class SearchWidget;
 	class SearchPlugin;
-	class SearchEngineList;
 	
 	
 	/**
@@ -51,7 +50,7 @@ namespace kt
 	{
 		Q_OBJECT
 	public:
-		SearchWidget(SearchPlugin* sp,SearchEngineList* sl);
+		SearchWidget(SearchPlugin* sp);
 		virtual ~SearchWidget();
 	
 		KMenu* rightClickMenu();
@@ -60,12 +59,14 @@ namespace kt
 		KUrl getCurrentUrl() const;
 		QString getSearchBarText() const;
 		int getSearchBarEngine() const;
+		void setSearchBarEngine(int engine);
 		
 		bool backAvailable() const;
 		
 	signals:
 		void enableBack(bool on);
 		void openNewTab(const KUrl & url);
+		void changeTitle(SearchWidget* w,const QString & title);
 	
 	public slots:
 		void search(const QString & text,int engine = 0);
@@ -75,7 +76,9 @@ namespace kt
 		void back();
 		void reload();
 		void onShutDown();
+		void home();
 		void restore(const KUrl & url,const QString & text,const QString & sb_text,int engine);
+		void onSearchRequested(const QString & text);
 	
 	private slots:
 		void onUrlHover(const QString & url);
