@@ -29,14 +29,16 @@
 namespace kt
 {
 
-	FeedWidget::FeedWidget(Feed* feed,FilterList* filters,SyndicationPlugin* plugin,QWidget* parent)
-			: QWidget(parent),feed(feed),filters(filters),plugin(plugin)
+	FeedWidget::FeedWidget(Feed* feed,FilterList* filters,SyndicationActivity* act,QWidget* parent)
+			: QWidget(parent),feed(feed),filters(filters),act(act)
 	{
 		setupUi(this);
 		connect(feed,SIGNAL(updated()),this,SLOT(updated()));
+		connect(feed,SIGNAL(feedRenamed(Feed*)),this,SLOT(onFeedRenamed(Feed*)));
 		connect(m_download,SIGNAL(clicked()),this,SLOT(downloadClicked()));
 		connect(m_refresh,SIGNAL(clicked()),this,SLOT(refreshClicked()));
 		connect(m_filters,SIGNAL(clicked()),this,SLOT(filtersClicked()));
+		connect(m_refresh_rate,SIGNAL(valueChanged(int)),this,SLOT(refreshRateChanged(int)));
 		
 		model = new FeedWidgetModel(feed,this);
 		m_item_list->setModel(model);
@@ -48,6 +50,7 @@ namespace kt
 				this,SLOT(selectionChanged(const QItemSelection&, const QItemSelection&)));
 		m_download->setEnabled(false);
 		m_url->setText(QString("<b>%1</b>").arg(feed->feedUrl().prettyUrl()));
+		m_refresh_rate->setValue(feed->refreshRate());
 		updated();
 	}
 
@@ -72,9 +75,15 @@ namespace kt
 		feed->refresh();
 	}
 	
+	void FeedWidget::refreshRateChanged(int v)
+	{
+		if (v > 0)
+			feed->setRefreshRate(v);
+	}
+	
 	void FeedWidget::filtersClicked()
 	{
-		ManageFiltersDlg dlg(feed,filters,plugin,this);
+		ManageFiltersDlg dlg(feed,filters,act,this);
 		if (dlg.exec() == QDialog::Accepted)
 		{
 			feed->save();
@@ -107,4 +116,11 @@ namespace kt
 		}
 		updateCaption(this,feed->title());
 	}
+	
+	
+	void FeedWidget::onFeedRenamed(kt::Feed* f)
+	{
+		updateCaption(this,f->displayName());
+	}
+
 }
