@@ -77,6 +77,7 @@ namespace kt
 		feed_view_menu = new KMenu(this);
 		feed_view_menu->addAction(ac->action("show_feed"));
 		feed_view_menu->addAction(ac->action("manage_filters"));
+		feed_view_menu->addAction(ac->action("edit_feed_name"));
 		feed_view_menu->addSeparator();
 		feed_view_menu->addAction(ac->action("add_feed"));
 		feed_view_menu->addAction(ac->action("remove_feed"));
@@ -100,12 +101,21 @@ namespace kt
 
 	void SyndicationTab::showFeedViewMenu(const QPoint & pos)
 	{
-		feed_view_menu->popup(feed_view->mapToGlobal(pos));
+		feed_view_menu->popup(feed_view->viewport()->mapToGlobal(pos));
 	}
 	
 	void SyndicationTab::showFilterViewMenu(const QPoint & pos)
 	{
-		filter_view_menu->popup(filter_view->mapToGlobal(pos));
+		filter_view_menu->popup(filter_view->viewport()->mapToGlobal(pos));
 	}
 
+	void SyndicationTab::loadState(KConfigGroup & g)
+	{
+		splitter->restoreState(g.readEntry("ver_splitter",QByteArray()));
+	}
+	
+	void SyndicationTab::saveState(KConfigGroup & g)
+	{
+		g.writeEntry("ver_splitter",splitter->saveState());
+	}
 }
