@@ -71,6 +71,7 @@ namespace kt
 	{
 		qsrand(time(0));
 		sock = new net::Socket(true,4);
+		content_generators.setAutoDelete(true);
 		addContentGenerator(new TorrentListGenerator(core,this));
 		addContentGenerator(new ChallengeGenerator(this));
 		addContentGenerator(new LoginHandler(this));
@@ -83,8 +84,11 @@ namespace kt
 		addContentGenerator(new SettingsGenerator(core,this));
 
 		QStringList dirList = KGlobal::dirs()->findDirs("data", "ktorrent/www");
-		rootDir = dirList.front();
-		Out(SYS_WEB|LOG_DEBUG) << "WWW Root Directory "<< rootDir <<endl;
+		if (!dirList.empty())
+		{
+			rootDir = dirList.front();
+			Out(SYS_WEB|LOG_DEBUG) << "WWW Root Directory "<< rootDir <<endl;
+		}
 		session.logged_in = false;
 		
 		ok = sock->bind(QString::null,port,true);
@@ -94,18 +98,22 @@ namespace kt
 			connect(notifier,SIGNAL(activated(int)),this,SLOT(slotAccept(int)));
 		}
 
-		skin_list = QDir(rootDir).entryList(QDir::Dirs);
-		skin_list.removeAll("common");
-		skin_list.removeAll(".");
-		skin_list.removeAll("..");
-		foreach (QString s,skin_list)
-			Out(SYS_WEB|LOG_DEBUG) << "skin: " << s << endl;
+		if (!rootDir.isEmpty())
+		{
+			skin_list = QDir(rootDir).entryList(QDir::Dirs);
+			skin_list.removeAll("common");
+			skin_list.removeAll(".");
+			skin_list.removeAll("..");
+			foreach (QString s,skin_list)
+				Out(SYS_WEB|LOG_DEBUG) << "skin: " << s << endl;
+		}
 	}
 	
 	HttpServer::~HttpServer()
 	{
 		sock->close();
 		delete sock;
+		qDeleteAll(clients);
 	}
 
 	QString HttpServer::skinDir() const
@@ -144,6 +152,7 @@ namespace kt
 		HttpClientHandler* handler = new HttpClientHandler(this,socket);
 		connect(handler,SIGNAL(closed()),this,SLOT(slotConnectionClosed()));
 		Out(SYS_WEB|LOG_NOTICE) << "connection from "<< addr.toString()  << endl;
+		clients.append(handler);
 	}
 
 	bool HttpServer::checkLogin(const QHttpRequestHeader & hdr,const QByteArray & data)
@@ -280,7 +289,9 @@ namespace kt
 	{
 		hdr.setValue("Server","KTorrent/" KT_VERSION_MACRO);
 		hdr.setValue("Date",DateTimeToString(QDateTime::currentDateTime().toUTC(),false));
-		hdr.setValue("Content-Type",content_type);
+		if (!content_type.isEmpty())
+			hdr.setValue("Content-Type",content_type);
+		
 		hdr.setValue("Connection","keep-alive");
 		if (with_session_info && session.sessionId && session.logged_in)
 		{
@@ -290,16 +301,11 @@ namespace kt
 	
 	void HttpServer::redirectToLoginPage(HttpClientHandler* hdlr)
 	{
-		HttpResponseHeader rhdr(301);
+		HttpResponseHeader rhdr(302);
 		setDefaultResponseHeaders(rhdr,"text/html",false);
-		rhdr.setValue("Location","/login.html");
-		QString path = skinDir() + "/login.html";
-		if (!hdlr->sendFile(rhdr,path))
-		{
-			HttpResponseHeader nhdr(404);
-			setDefaultResponseHeaders(nhdr,"text/html",false);
-			hdlr->send404(nhdr,path);
-		}
+		rhdr.setValue("Location","login.html");
+		rhdr.setValue("Content-Length","0");
+		hdlr->sendResponse(rhdr);
 		Out(SYS_WEB|LOG_NOTICE) << "Redirecting to /login.html" << endl;
 	}
 	
@@ -307,6 +313,14 @@ namespace kt
 	
 	void HttpServer::handleGet(HttpClientHandler* hdlr,const QHttpRequestHeader & hdr)
 	{
+		if (rootDir.isEmpty())
+		{
+			HttpResponseHeader rhdr(500);
+			setDefaultResponseHeaders(rhdr,"text/html",false);
+			hdlr->send500(rhdr,i18n("Cannot find webinterface skins!"));
+			return;
+		}
+		
 		QString file = hdr.path();
 		if (file == "/")
 			file = "/login.html";
@@ -464,12 +478,13 @@ namespace kt
 	{
 		HttpResponseHeader rhdr(500);
 		setDefaultResponseHeaders(rhdr,"text/html",false);
-		hdlr->send500(rhdr);
+		hdlr->send500(rhdr,i18n("Unsupported HTTP method"));
 	}
 	
 	void HttpServer::slotConnectionClosed()
 	{
 		HttpClientHandler* client = (HttpClientHandler*)sender();
+		clients.removeAll(client);
 		client->deleteLater();
 	}
 
